<?php

namespace Drupal\Tests\webform_product\Functional;

use Drupal\Tests\BrowserTestBase;

/**
 * Tests the loading of webform product.
 *
 * @group WebformProduct
 */
class WebformProductUiTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Module list.
   *
   * @var array
   */
  public static $modules = [
    'commerce',
    'commerce_price',
    'commerce_store',
    'commerce_number_pattern',
    'commerce_product',
    'commerce_cart',
    'commerce_checkout',
    'commerce_payment',
    'commerce_payment_example',
    'field',
    'link',
    'user',
    'webform',
    'webform_ui',
    'webform_product',
    'webform_product_test',
  ];

  /**
   * Admin user.
   *
   * @var \Drupal\user\Entity\User|false
   */
  private $adminUser;

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function setUp() {
    parent::setUp();

    // Create the user and login.
    $this->adminUser = $this->createUser([], NULL, TRUE);
    $this->drupalLogin($this->adminUser);
  }

  /**
   * Test webform product handler settings page.
   */
  public function testWebformProductSettingsPage() {
    $this->drupalGet('admin/structure/webform/manage/webform_payment_example/handlers/webform_product/edit');
    $this->assertFieldsByValue($this->xpath("//input[@id = 'edit-settings-commerce-order-item-title']"), '[webform_submission:source-entity:title]');
    $this->assertOptionSelected('settings[commerce][store]', 'Test store');
    $this->assertOptionSelected('settings[commerce][order_type]', 'Default');
    $this->assertOptionSelected('settings[commerce][order_item_type]', 'Webform');
    $this->assertOptionSelected('settings[commerce][checkout_step]', 'review');
    $this->assertOptionSelected('settings[commerce][payment_gateway]', 'example');
    $this->assertOptionSelected('settings[order_data][payment_price]', '- None -');
    $this->assertOptionSelected('settings[field_mapping][field_payment_status]', 'payment_status');
    $this->assertOptionSelected('settings[field_mapping][field_order_id]', 'order_id');
    $this->assertOptionSelected('settings[field_mapping][field_order_url]', 'order_url');
    $this->assertOptionSelected('settings[field_mapping][field_total_price]', 'total_price');

    // Click add another link.
    $this->click('#edit-submit');

    // Ensure we saved correctly.
    $this->assertText('The webform handler was successfully updated.');
  }

  /**
   * Test: Change product price.
   */
  public function testWebformProductChangePrice() {
    $this->drupalGet('admin/structure/webform/manage/webform_payment_example/element/name/edit');
    $this->assertFieldsByValue($this->xpath("//input[@id = 'edit-price']"), 100);

    $edit = [
      'price' => 50,
    ];
    $this->submitForm($edit, 'Save', 'webform-ui-element-form');

    // Ensure we saved correctly.
    $this->assertText('Name has been updated.');

    $this->drupalGet('admin/structure/webform/manage/webform_payment_example/element/name/edit');
    $this->assertFieldsByValue($this->xpath("//input[@id = 'edit-price']"), 50);
  }

  /**
   * Test: Order process.
   */
  public function testWebformProductOrderProcess() {
    // Set order default validation:
    $this->drupalGet('admin/commerce/config/order-types/default/edit');
    $edit = [
      'edit-workflow' => 'order_default_validation',
    ];
    $this->submitForm($edit, 'Save', 'commerce-order-type-edit-form');

    // Send webform:
    $this->drupalGet('form/webform-payment-example');

    $edit = [
      'edit-name' => 'Test Elek',
    ];
    $this->submitForm(
      $edit,
      'Submit',
      'webform-submission-webform-payment-example-add-form'
    );

    // Ensure we saved correctly. Check checkout process.
    $this->assertText('[webform_submission:source-entity:title]');
    $this->assertRaw(
      '<div class="order-total-line order-total-line__subtotal">
    <span class="order-total-line-label">Subtotal </span><span class="order-total-line-value">€100.00</span>
  </div>'
    );
    $this->assertRaw(
      '<div class="order-total-line order-total-line__total">
    <span class="order-total-line-label">Total </span><span class="order-total-line-value">€100.00</span>
  </div>'
    );

    $this->assertFieldsByValue(
      $this->xpath("//input[@id = 'edit-actions-next']"),
      'Pay and complete purchase'
    );

    $checkout_url = $this->getUrl();

    // Check webform submission.
    $this->drupalGet(
      'admin/structure/webform/manage/webform_payment_example/results/submissions'
    );
    $this->assertText('1 (draft)');
    $this->assertText('/admin/commerce/orders/1');

    // Continue checkout:
    $this->drupalGet($checkout_url);
    $this->submitForm(
      [],
      'Pay and complete purchase',
      'commerce-checkout-flow-multistep-default'
    );
    $this->assertText(
      'Please wait while you are redirected to the payment server. If nothing happens within 10 seconds, please click on the button below.'
    );

    $checkout_url = $this->getUrl();

    // Check webform submission.
    $this->drupalGet(
      'admin/structure/webform/manage/webform_payment_example/results/submissions'
    );
    $this->assertText('1 (draft)');
    $this->assertText('/admin/commerce/orders/1');

    // Continue checkout:
    $this->drupalGet($checkout_url);
    $this->submitForm(
      [],
      'Proceed to Example',
      'commerce-checkout-flow-multistep-default'
    );
    $this->assertText('Complete');

    // Validate order:
    $this->drupalGet('/admin/commerce/orders/1');
    $this->assertText('Validation');
    $this->submitForm(
      [],
      'Validate order',
      'state-machine-transition-form-commerce-order-state-1'
    );
    $this->assertText('Completed');

    // Check webform submission.
    $this->drupalGet(
      'admin/structure/webform/manage/webform_payment_example/results/submissions'
    );
    $this->assertNoText('1 (draft)');
  }

}
